@@ -15,7 +15,7 @@ import sys
 import time
 import traceback
 
-from test_psutil import (get_test_subprocess, reap_children, unittest)
+from test_psutil import WINDOWS, get_test_subprocess, reap_children, unittest
 
 try:
     import wmi
@@ -28,16 +28,18 @@ except ImportError:
     win32api = win32con = None
 
 from psutil._compat import PY3, callable, long
-from psutil._pswindows import ACCESS_DENIED_SET
-import _psutil_windows
 import psutil
 
 
+cext = psutil._psplatform.cext
+
+
 def wrap_exceptions(fun):
     def wrapper(self, *args, **kwargs):
         try:
             return fun(self, *args, **kwargs)
         except OSError as err:
+            from psutil._pswindows import ACCESS_DENIED_SET
             if err.errno in ACCESS_DENIED_SET:
                 raise psutil.AccessDenied(None, None)
             if err.errno == errno.ESRCH:
@@ -46,6 +48,7 @@ def wrap_exceptions(fun):
     return wrapper
 
 
+@unittest.skipUnless(WINDOWS, "not a Windows system")
 class WindowsSpecificTestCase(unittest.TestCase):
 
     @classmethod
@@ -113,7 +116,9 @@ class WindowsSpecificTestCase(unittest.TestCase):
     def test_process_exe(self):
         w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
         p = psutil.Process(self.pid)
-        self.assertEqual(p.exe(), w.ExecutablePath)
+        # Note: wmi reports the exe as a lower case string.
+        # Being Windows paths case-insensitive we ignore that.
+        self.assertEqual(p.exe().lower(), w.ExecutablePath.lower())
 
     @unittest.skipIf(wmi is None, "wmi module is not installed")
     def test_process_cmdline(self):
@@ -163,7 +168,7 @@ class WindowsSpecificTestCase(unittest.TestCase):
 
     # --- psutil namespace functions and constants tests
 
-    @unittest.skipUnless(hasattr(os, 'NUMBER_OF_PROCESSORS'),
+    @unittest.skipUnless('NUMBER_OF_PROCESSORS' in os.environ,
                          'NUMBER_OF_PROCESSORS env var is not available')
     def test_cpu_count(self):
         num_cpus = int(os.environ['NUMBER_OF_PROCESSORS'])
@@ -257,8 +262,6 @@ class WindowsSpecificTestCase(unittest.TestCase):
         failures = []
         for name in dir(psutil.Process):
             if name.startswith('_') \
-                or name.startswith('set_') \
-                or name.startswith('get')  \
                 or name in ('terminate', 'kill', 'suspend', 'resume',
                             'nice', 'send_signal', 'wait', 'children',
                             'as_dict'):
@@ -281,14 +284,15 @@ class WindowsSpecificTestCase(unittest.TestCase):
             self.fail('\n' + '\n'.join(failures))
 
 
+@unittest.skipUnless(WINDOWS, "not a Windows system")
 class TestDualProcessImplementation(unittest.TestCase):
     fun_names = [
         # function name, tolerance
         ('proc_cpu_times', 0.2),
         ('proc_create_time', 0.5),
         ('proc_num_handles', 1),  # 1 because impl #1 opens a handle
-        ('proc_io_counters', 0),
         ('proc_memory_info', 1024),  # KB
+        ('proc_io_counters', 0),
     ]
 
     def test_compare_values(self):
@@ -302,7 +306,7 @@ class TestDualProcessImplementation(unittest.TestCase):
         def assert_ge_0(obj):
             if isinstance(obj, tuple):
                 for value in obj:
-                    self.assertGreaterEqual(value, 0)
+                    self.assertGreaterEqual(value, 0, msg=obj)
             elif isinstance(obj, (int, long, float)):
                 self.assertGreaterEqual(obj, 0)
             else:
@@ -320,55 +324,76 @@ class TestDualProcessImplementation(unittest.TestCase):
                         diff = abs(a - b)
                         self.assertLessEqual(diff, tolerance)
 
+        from psutil._pswindows import ntpinfo
         failures = []
-        for name, tolerance in self.fun_names:
-            meth1 = wrap_exceptions(getattr(_psutil_windows, name))
-            meth2 = wrap_exceptions(getattr(_psutil_windows, name + '_2'))
-            for p in psutil.process_iter():
+        for p in psutil.process_iter():
+            try:
+                nt = ntpinfo(*cext.proc_info(p.pid))
+            except psutil.NoSuchProcess:
+                continue
+            assert_ge_0(nt)
+
+            for name, tolerance in self.fun_names:
                 if name == 'proc_memory_info' and p.pid == os.getpid():
                     continue
-                #
-                try:
-                    ret1 = meth1(p.pid)
-                except psutil.NoSuchProcess:
+                if name == 'proc_create_time' and p.pid in (0, 4):
                     continue
-                except psutil.AccessDenied:
-                    ret1 = None
-                #
+                meth = wrap_exceptions(getattr(cext, name))
                 try:
-                    ret2 = meth2(p.pid)
-                except psutil.NoSuchProcess:
-                    # this is supposed to fail only in case of zombie process
-                    # never for permission error
+                    ret = meth(p.pid)
+                except (psutil.NoSuchProcess, psutil.AccessDenied):
                     continue
-
                 # compare values
                 try:
-                    if ret1 is None:
-                        assert_ge_0(ret2)
-                    else:
-                        compare_with_tolerance(ret1, ret2, tolerance)
-                        assert_ge_0(ret1)
-                        assert_ge_0(ret2)
+                    if name == 'proc_cpu_times':
+                        compare_with_tolerance(ret[0], nt.user_time, tolerance)
+                        compare_with_tolerance(ret[1],
+                                               nt.kernel_time, tolerance)
+                    elif name == 'proc_create_time':
+                        compare_with_tolerance(ret, nt.create_time, tolerance)
+                    elif name == 'proc_num_handles':
+                        compare_with_tolerance(ret, nt.num_handles, tolerance)
+                    elif name == 'proc_io_counters':
+                        compare_with_tolerance(ret[0], nt.io_rcount, tolerance)
+                        compare_with_tolerance(ret[1], nt.io_wcount, tolerance)
+                        compare_with_tolerance(ret[2], nt.io_rbytes, tolerance)
+                        compare_with_tolerance(ret[3], nt.io_wbytes, tolerance)
+                    elif name == 'proc_memory_info':
+                        try:
+                            rawtupl = cext.proc_memory_info_2(p.pid)
+                        except psutil.NoSuchProcess:
+                            continue
+                        compare_with_tolerance(ret, rawtupl, tolerance)
                 except AssertionError:
                     trace = traceback.format_exc()
                     msg = '%s\npid=%s, method=%r, ret_1=%r, ret_2=%r' % (
-                          trace, p.pid, name, ret1, ret2)
+                        trace, p.pid, name, ret, nt)
                     failures.append(msg)
                     break
+
         if failures:
             self.fail('\n\n'.join(failures))
 
+    def test_compare_name_exe(self):
+        for p in psutil.process_iter():
+            try:
+                a = os.path.basename(p.exe())
+                b = p.name()
+            except (psutil.NoSuchProcess, psutil.AccessDenied):
+                pass
+            else:
+                self.assertEqual(a, b)
+
     def test_zombies(self):
         # test that NPS is raised by the 2nd implementation in case a
         # process no longer exists
         ZOMBIE_PID = max(psutil.pids()) + 5000
         for name, _ in self.fun_names:
-            meth = wrap_exceptions(getattr(_psutil_windows, name))
+            meth = wrap_exceptions(getattr(cext, name))
             self.assertRaises(psutil.NoSuchProcess, meth, ZOMBIE_PID)
 
 
-def test_main():
+def main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(WindowsSpecificTestCase))
     test_suite.addTest(unittest.makeSuite(TestDualProcessImplementation))
@@ -376,5 +401,5 @@ def test_main():
     return result.wasSuccessful()
 
 if __name__ == '__main__':
-    if not test_main():
+    if not main():
         sys.exit(1)
